@@ -4,24 +4,38 @@ from .helper import (
     bitfield,
     buffer_end,
     buffer_start,
+    bytestr,
     checksum_end,
     checksum_field,
     checksum_start,
     const,
     const_into,
+    crypt,
+    crypt_end,
     eval_into,
-    hook_end,
     packing,
     probe,
-    string,
     tell,
     tell_into,
+    text,
     unpacking,
     validate,
     varlist,
+    vartext,
     virtual,
 )
-from .special import action, align, alignto, hook, padding, seek, skip
+from .special import (
+    action,
+    align,
+    alignto,
+    hook,
+    hook_end,
+    io,
+    io_end,
+    padding,
+    seek,
+    skip,
+)
 from .standard import built, field, subfield
 from .wrapper import adapter, cond, repeat, switch
 
@@ -34,29 +48,35 @@ __all__ = [
     "buffer_end",
     "buffer_start",
     "built",
+    "bytestr",
     "checksum_end",
     "checksum_field",
     "checksum_start",
     "cond",
     "const",
     "const_into",
+    "crypt",
+    "crypt_end",
     "eval_into",
     "field",
     "hook",
     "hook_end",
+    "io",
+    "io_end",
     "packing",
     "padding",
     "probe",
     "repeat",
     "seek",
     "skip",
-    "string",
     "subfield",
     "switch",
     "tell",
     "tell_into",
+    "text",
     "unpacking",
     "validate",
     "varlist",
+    "vartext",
     "virtual",
 ]
