@@ -7,8 +7,7 @@ from functools import lru_cache
 from io import BytesIO
 from typing import IO, Any, Dict, List, Optional, Tuple, Type, TypeVar, Union
 
-from .context import Container, Context
-from .types import Config, FieldMeta, FieldType, T
+from .types import Config, Container, Context, FieldMeta, FieldType, T
 from .utils.config import datastruct_get_config
 from .utils.const import ARRAYS, BYTES, EXCEPTIONS
 from .utils.context import (
@@ -18,6 +17,7 @@ from .utils.context import (
     ctx_write,
     evaluate,
     hook_apply,
+    io_apply,
 )
 from .utils.fields import (
     field_decode,
@@ -27,6 +27,7 @@ from .utils.fields import (
     field_get_default,
     field_get_meta,
     field_get_padding,
+    field_get_type,
     field_switch_base,
     field_validate,
 )
@@ -40,6 +41,28 @@ class DataStruct:
         for field, meta, value in self.fields():
             field_validate(field, meta)
 
+            if value != Ellipsis:
+                # try to map simple default values
+                field_type, _ = field_get_type(field)
+                if field_type is not None:
+                    value = field_decode(value, field_type)
+                    if not isinstance(value, field_type):
+                        if meta.adapter:
+                            # try to adapt default values
+                            try:
+                                value = meta.adapter.decode(value, None)
+                            except Exception:
+                                raise ValueError(
+                                    f"Couldn't build the default value of {field.name} - the adapter "
+                                    "might be requiring the context. Pass 'default' value "
+                                    "resulting *after* applying the adapter"
+                                )
+                        else:
+                            raise TypeError(
+                                f"Wrong field type - expected {field_type}, found default {type(value)}"
+                            )
+                    self.__setattr__(field.name, value)
+
             # accept special fields and those already having a value
             if value != Ellipsis or not meta.public:
                 continue
@@ -139,6 +162,12 @@ class DataStruct:
             hook_apply(ctx, meta)
             return Ellipsis
 
+        if meta.ftype == FieldType.IO:
+            if ctx.G.sizing:
+                return Ellipsis
+            io_apply(ctx, meta)
+            return Ellipsis
+
         if meta.ftype == FieldType.REPEAT:
             # repeat() field - value type must be List
             if not isinstance(value, ARRAYS):
@@ -179,7 +208,14 @@ class DataStruct:
                     break
 
                 if not base_meta.builder:
-                    item = next(items_iter)
+                    try:
+                        item = next(items_iter)
+                    except StopIteration:
+                        raise ValueError(
+                            "No more items to pack! "
+                            "Check the return value of 'when' and 'last'. "
+                            "Perhaps the list is missing the last element?"
+                        )
                 else:
                     item = Ellipsis
                 item = self._write_field(ctx, base_field, base_meta, item)
@@ -203,6 +239,9 @@ class DataStruct:
 
         if meta.ftype == FieldType.COND:
             if evaluate(ctx, meta.condition) is False:
+                if meta.if_not is not Ellipsis:
+                    value = evaluate(ctx, meta.if_not)
+                    return value
                 return Ellipsis
             return self._write_field(ctx, *field_get_base(meta), value)
 
@@ -266,6 +305,10 @@ class DataStruct:
             hook_apply(ctx, meta)
             return Ellipsis
 
+        if meta.ftype == FieldType.IO:
+            io_apply(ctx, meta)
+            return Ellipsis
+
         if meta.ftype == FieldType.REPEAT:
             # repeat() field - value type must be List
             if not issubclass(field.type, ARRAYS):
@@ -281,7 +324,7 @@ class DataStruct:
                 if evaluate(ctx, meta.when) is False:
                     break
 
-                item = cls._read_value(ctx, base_meta, base_field.type)
+                item = cls._read_field(ctx, base_field, base_meta)
                 items.append(item)
 
                 # provide another value 'item' to context lambdas in 'last'
@@ -310,7 +353,10 @@ class DataStruct:
     def pack(
         self,
         io: Optional[IO[bytes]] = None,
+        field_names: Union[str, List[str]] = None,
         parent: Union[Context, "DataStruct", None] = None,
+        ctx_out: List[Context] = None,
+        tell_offset: int = 0,
         **kwargs,
     ) -> Optional[bytes]:
         sizing = isinstance(io, SizingIO)
@@ -327,17 +373,33 @@ class DataStruct:
             parent = build_context(glob, None, self.config())
             parent.self = parent_obj
 
+        if not field_names:
+            field_names = []
+        if isinstance(field_names, str):
+            field_names = [field_names]
+
         fields = self.fields()
         ctx = build_context(glob, parent, self.config(), **kwargs)
         ctx.self = self
+        if ctx_out is not None:
+            ctx_out.append(ctx)
+        if tell_offset:
+            ctx.P.seek(tell_offset)
         field_name = type(self).__name__
         try:
+            field_found = not field_names
             for field, meta, _ in fields:
+                if field_names and field.name not in field_names:
+                    continue
+                field_found = True
                 field_name = f"{type(self).__name__}.{field.name}"
                 # print(f"Packing {meta.ftype.name} '{field_name}'")
-                value = self._write_field(ctx, field, meta, ctx[field.name])
+                value = self._write_field(ctx, field, meta, getattr(self, field.name))
                 if value is not Ellipsis and meta.public:
                     setattr(self, field.name, value)
+            if not field_found:
+                # after packing, the field must have been found
+                raise ValueError(f"No such field(s): {field_names}")
         except EXCEPTIONS as e:
             if ctx.G.sizing:
                 suffix = f"; while sizing '{field_name}'"
@@ -354,6 +416,7 @@ class DataStruct:
         cls: Type["DS"],
         io: Union[IO[bytes], bytes],
         parent: Union[Context, "DataStruct", None] = None,
+        ctx_out: List[Context] = None,
         **kwargs,
     ) -> "DS":
         if isinstance(parent, Context):
@@ -371,6 +434,8 @@ class DataStruct:
         values = Container()
         ctx = build_context(glob, parent, cls.config(), **kwargs)
         ctx.self = values
+        if ctx_out is not None:
+            ctx_out.append(ctx)
         field_name = cls.__name__
         try:
             for field, meta in fields:
@@ -391,11 +456,21 @@ class DataStruct:
 
     def sizeof(
         self,
+        field_names: Union[str, List[str]] = None,
         parent: Union[Context, "DataStruct", None] = None,
+        ctx_out: List[Context] = None,
+        tell_offset: int = 0,
         **kwargs,
     ) -> int:
         io = SizingIO()
-        self.pack(io=io, parent=parent, **kwargs)
+        self.pack(
+            io=io,
+            field_names=field_names,
+            parent=parent,
+            ctx_out=ctx_out,
+            tell_offset=tell_offset,
+            **kwargs,
+        )
         return io.size
 
     def fields(self) -> List[Tuple[Field, FieldMeta, Any]]:
