@@ -5,8 +5,7 @@ from enum import Enum
 from io import SEEK_CUR
 from typing import Any, Optional, Tuple
 
-from ..context import Context
-from ..types import Config, FieldMeta, FieldType
+from ..types import Config, Context, FieldMeta, FieldType
 from .const import ARRAYS
 from .context import evaluate
 from .fmt import fmt_check
@@ -14,8 +13,6 @@ from .misc import pad_up, repstr
 
 
 def field_encode(v: Any) -> Any:
-    if isinstance(v, str):
-        return v.encode()
     if isinstance(v, int):
         return v
     if isinstance(v, Enum):
@@ -24,8 +21,6 @@ def field_encode(v: Any) -> Any:
 
 
 def field_decode(v: Any, cls: type) -> Any:
-    if issubclass(cls, str):
-        return v.decode()
     if issubclass(cls, Enum):
         return cls(v)
     return v
@@ -35,6 +30,8 @@ def field_get_type(field: Field) -> Tuple[type, Optional[type]]:
     field_type = field.type
     if field_type is Ellipsis:
         return field_type, None
+    if field_type is Any:
+        return None, None
     if hasattr(field_type, "__origin__"):
         field_type = field.type.__origin__
     if not isinstance(field_type, type):
@@ -204,10 +201,11 @@ def field_validate(field: Field, meta: FieldMeta) -> None:
         if item_type is None:
             # var: ... = repeat()(...)
             raise ValueError("Can't use repeat() for a non-list field")
-        if base_meta.ftype != FieldType.FIELD:
+        if base_meta.ftype not in [FieldType.FIELD, FieldType.COND, FieldType.SWITCH]:
             # var: ... = repeat()(padding(...))
             raise ValueError(
-                "Only field(), subfield() and built() can be used with repeat()"
+                "Only field(), subfield(), built(), cond() and switch() "
+                "can be used with repeat()"
             )
         if base_meta.builder and not base_meta.always:
             # var: ... = repeat()(built(..., always=False))
